/*** Copyright: OGIHARA Takeshi, May 2008 ***/

#import <Cocoa/Cocoa.h>
#import "UroServer.h"
#import "UroView.h"
#import "UroRemotePiece.h"

@implementation UroServer

- (void)removePieceAt:(UroPoint)pos
{
    id rmv;
    if ((rmv = uBoard[pos.y][pos.x]) != nil) {
        uBoard[pos.y][pos.x] = nil;
        [uView drawFrom:pos to:pos];
        [proxies removeObject: rmv];
    }
}

/* Local Method */
- (void)removeInvalidConnection:(NSNotification *)notification
{
    UroPoint pt;
    int x, y;
    id obj, rmv = nil;
    id invalid = [notification object];
    for (obj in proxies) {
        if ([obj connectionForProxy] == invalid) {
            rmv = [obj retain];
            [proxies removeObject: obj];
            break;
        }
    }
    if (rmv == nil) return;
    for (y = 0; y < BHeight; y++)
        for (x = 0; x < BWidth; x++)
            if (uBoard[y][x] == rmv) break;
    if (y >= BHeight) return;
    [rmv release];
    pt.x = x;
    pt.y = y;
    [self removePieceAt:pt];
}

/* Local Method */
- (void)setupServer
{
    id theConnection = [NSConnection defaultConnection];
    [theConnection setRootObject:self];
    if ([theConnection registerName: ServerName] == NO) {
        NSLog(@"Can't register server\n");
        return;
    }
    [[NSNotificationCenter defaultCenter] addObserver:self
        selector:@selector(removeInvalidConnection:)
        name:NSConnectionDidDieNotification object:nil];
}

- (BOOL)registerAsRemoteClient:(id)client
{
    if (!inAction)
        return NO;
    [client setProtocolForProxy:@protocol(UroRemoteClient)];
    [client setBoardWidth:BWidth height:BHeight];
    [proxies addObject:client];
    [self resetBoard:self];
    return YES;
}

/* Application Delegate Message */
- (void)applicationDidFinishLaunching:(NSNotification *)note
{
    [self setupServer];
    [super applicationDidFinishLaunching:note];
}

@end
